# perl
#
# Class Report::Porf::Table::Simple
#
# Use Configurator or Framework of namespace Report::Porf to create Instances,
# that export data as text, html, csv, LaTeX, for wikis and Excel
#
# Ralf Peine, Sat Feb 01 08:15:52 2014
#
# More documentation at the end of file
#------------------------------------------------------------------------------

$VERSION = "0.950";

#------------------------------------------------------------------------------
#
# Easy create lists like this as text, html, csv, ...: 
#
# *============+======================+=========+=======+======================*
# @ Count      @ Prename              @ Surname @ Age   @ TimeStamp            @
# *------------+----------------------+---------+-------+----------------------*
# | 1          | Vorname 1            | Name 1  | 10    | 0.002329             |
# | 2          | Vorname 2            | Name 2  | 20    | 0.003106             |
# | 3          | Vorname 3            | Name 3  | 30    | 0.003822             |
# | 4          | Vorname 4            | Name 4  | 40    | 0.004533             |
# | 5          | Vorname 5            | Name 5  | 50    | 0.005235             |
# | 6          | Vorname 6            | Name 6  | 60    | 0.005944             |
# | 7          | Vorname 7            | Name 7  | 70    | 0.006656             |
# | 8          | Vorname 8            | Name 8  | 80    | 0.007362             |
# | 9          | Vorname 9            | Name 9  | 90    | 0.008069             |
# | 10         | Vorname 10           | Name 10 | 100   | 0.008779             |
# *============+======================+=========+=======+======================*
#
# # Time needed for export of 10 data lines: 0.001954
#
#------------------------------------------------------------------------------

use strict;
use warnings;

#--------------------------------------------------------------------------------
#
#  Report::Porf::Table::Simple;
#
#--------------------------------------------------------------------------------

package Report::Porf::Table::Simple;

use Carp;
use FileHandle;

use Report::Porf::Util;

#--------------------------------------------------------------------------------
#
#  Creation / Filling Of Instances
#
#--------------------------------------------------------------------------------

# --- Create Instance -----------------
sub new
{
    my $caller = $_[0];
    my $class  = ref($caller) || $caller;
    
    # let the class go
    my $self = {};
    bless $self, $class;

    $self->_init();
    
    return $self;
}

# --- initialise instance -------------
sub _init {
    my ($self,        # instance_ref
        ) = @_;

    $self->SetDefaultColumnWidth (0);
    $self->SetMaxColWidth        (0);
    $self->SetMaxColumnIdx       (-2);
    $self->SetFormat             ('?');

    $self->SetColumnWidthsRef([]);
    $self->SetHeaderTextsRef ([]);
    
    $self->SetFileStart  ('');
    $self->SetPageStart  ('');
    $self->SetTableStart ('');

    $self->SetHeaderRowStart ('');
    $self->SetHeaderStart    ('');
    $self->SetHeaderEnd      ('');
    $self->SetHeaderRowEnd   ('');

    $self->SetRowStart  ('');
    $self->SetCellStart ('');
    $self->SetCellEnd   ('');
    $self->SetRowEnd    ('');

    $self->SetTableEnd ('');
    $self->SetPageEnd  ('');
    $self->SetFileEnd  ('');

    $self->SetDefaultAlign ('');

    $self->SetHeaderLine     ('');
    $self->SetSeparatorLine  ('');
    $self->SetBoldHeaderLine ('');

    $self->SetBold    ('');
    $self->SetItalics ('');
    $self->SetLeft    ('');
    $self->SetRight   ('');
    $self->SetCenter  ('');

    $self->SetHorizontalSeparationStart           ('');
    $self->SetHorizontalSeparationEnd             ('');
    $self->SetHorizontalSeparationColumnSeparator ('');
    $self->SetHorizontalSeparationChar            ('');
    $self->SetHorizontalSeparationBoldChar        ('');

    # $self->SetConfigureColumnAction   (sub {});
    # $self->SetConfigureCompleteAction (sub {});
    # $self->SetCellOutputAction        (sub {});
    # $self->SetHeaderOutputAction      (sub {});
    # $self->SetRowOutputAction         (sub {});
    # $self->SetStartTableOutputAction  (sub {});
    # $self->SetEndTableOutputAction    (sub {});
    # $self->SetRowGroupChangesAction   (sub {});

    $self->SetVerbose (0);

    my @CellOutputActions;
    $self->{CellOutputActions} = \@CellOutputActions;
}

#--------------------------------------------------------------------------------
#
#  Attributes
#
#--------------------------------------------------------------------------------

# Generated by peiner from CreateGetSetter.pl at Fri Apr 19 08:27:27 2013

# --- Format ---------------------------------------------------------------

sub SetFormat {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set Format after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{Format} = $value;
}

sub GetFormat {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{Format};
}

# --- Check, if given format is used for export ---
sub IsFormat {
    my ($self,        # instance_ref
        $format       # format to compare with
        ) = @_;
    
    return lc ($self->{Format}) eq lc($format) ? 1: 0;
}

# --- MaxColWidth ---------------------------------------------------------------

sub SetMaxColWidth {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set MaxColWidth after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{MaxColWidth} = $value;
}

sub GetMaxColWidth {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{MaxColWidth};
}

# --- DefaultColumnWidth ---------------------------------------------------------------

sub SetDefaultColumnWidth {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set DefaultColumnWidth after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{DefaultColumnWidth} = $value;
}

sub GetDefaultColumnWidth {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{DefaultColumnWidth};
}

# --- DefaultAlign ---------------------------------------------------------------

sub SetDefaultAlign {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set DefaultAlign after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{DefaultAlign} = $value;
}

sub GetDefaultAlign {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{DefaultAlign};
}

# --- MaxColumnIdx ---------------------------------------------------------------

sub SetMaxColumnIdx {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set MaxColumnIdx after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{MaxColumnIdx} = $value;
}

sub GetMaxColumnIdx {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{MaxColumnIdx};
}

# --- ColumnWidthsRef ---------------------------------------------------------------

sub SetColumnWidthsRef {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set ColumnWidthsRef after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{ColumnWidthsRef} = $value;
}

sub GetColumnWidthsRef {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{ColumnWidthsRef};
}

# --- HeaderTextsRef ---------------------------------------------------------------

sub SetHeaderTextsRef {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set HeaderTextsRef after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{HeaderTextsRef} = $value;
}

sub GetHeaderTextsRef {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{HeaderTextsRef};
}

# --- FileStart ---------------------------------------------------------------

sub SetFileStart {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set FileStart after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{FileStart} = $value;
}

sub GetFileStart {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{FileStart};
}

# --- FileEnd ---------------------------------------------------------------

sub SetFileEnd {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set FileEnd after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{FileEnd} = $value;
}

sub GetFileEnd {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{FileEnd};
}

# --- TableStart ---------------------------------------------------------------

sub SetTableStart {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set TableStart after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{TableStart} = $value;
}

sub GetTableStart {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{TableStart};
}

# --- TableEnd ---------------------------------------------------------------

sub SetTableEnd {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set TableEnd after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{TableEnd} = $value;
}

sub GetTableEnd {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{TableEnd};
}

# --- HeaderRowStart ---------------------------------------------------------------

sub SetHeaderRowStart {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set HeaderRowStart after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{HeaderRowStart} = $value;
}

sub GetHeaderRowStart {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{HeaderRowStart};
}

# --- HeaderRowEnd ---------------------------------------------------------------

sub SetHeaderRowEnd {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set HeaderRowEnd after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{HeaderRowEnd} = $value;
}

sub GetHeaderRowEnd {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{HeaderRowEnd};
}

# --- PageStart ---------------------------------------------------------------

sub SetPageStart {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set PageStart after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{PageStart} = $value;
}

sub GetPageStart {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{PageStart};
}

# --- PageEnd ---------------------------------------------------------------

sub SetPageEnd {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set PageEnd after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{PageEnd} = $value;
}

sub GetPageEnd {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{PageEnd};
}

# --- RowStart ---------------------------------------------------------------

sub SetRowStart {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set RowStart after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{RowStart} = $value;
}

sub GetRowStart {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{RowStart};
}

# --- RowEnd ---------------------------------------------------------------

sub SetRowEnd {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set RowEnd after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{RowEnd} = $value;
}

sub GetRowEnd {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{RowEnd};
}

# --- HeaderStart ---------------------------------------------------------------

sub SetHeaderStart {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set HeaderStart after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{HeaderStart} = $value;
}

sub GetHeaderStart {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{HeaderStart};
}

# --- HeaderEnd ---------------------------------------------------------------

sub SetHeaderEnd {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set HeaderEnd after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{HeaderEnd} = $value;
}

sub GetHeaderEnd {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{HeaderEnd};
}

# --- CellStart ---------------------------------------------------------------

sub SetCellStart {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set CellStart after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{CellStart} = $value;
}

sub GetCellStart {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{CellStart};
}

# --- CellEnd ---------------------------------------------------------------

sub SetCellEnd {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set CellEnd after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{CellEnd} = $value;
}

sub GetCellEnd {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{CellEnd};
}

# --- HeaderLine ---------------------------------------------------------------

sub SetHeaderLine {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set HeaderLine after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{HeaderLine} = $value;
}

sub GetHeaderLine {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{HeaderLine};
}

# --- SeparatorLine ---------------------------------------------------------------

sub SetSeparatorLine {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set SeparatorLine after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{SeparatorLine} = $value;
}

sub GetSeparatorLine {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{SeparatorLine};
}

# --- BoldHeaderLine ---------------------------------------------------------------

sub SetBoldHeaderLine {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set BoldHeaderLine after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{BoldHeaderLine} = $value;
}

sub GetBoldHeaderLine {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{BoldHeaderLine};
}

# --- Bold ---------------------------------------------------------------

sub SetBold {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set Bold after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{Bold} = $value;
}

sub GetBold {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{Bold};
}

# --- Italics ---------------------------------------------------------------

sub SetItalics {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set Italics after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{Italics} = $value;
}

sub GetItalics {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{Italics};
}

# --- Left ---------------------------------------------------------------

sub SetLeft {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set Left after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{Left} = $value;
}

sub GetLeft {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{Left};
}

# --- Right ---------------------------------------------------------------

sub SetRight {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set Right after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{Right} = $value;
}

sub GetRight {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{Right};
}

# --- Center ---------------------------------------------------------------

sub SetCenter {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set Center after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{Center} = $value;
}

sub GetCenter {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{Center};
}

# --- HorizontalSeparationStart ---------------------------------------------------------------

sub SetHorizontalSeparationStart {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set HorizontalSeparationStart after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{HorizontalSeparationStart} = $value;
}

sub GetHorizontalSeparationStart {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{HorizontalSeparationStart};
}

# --- HorizontalSeparationEnd ---------------------------------------------------------------

sub SetHorizontalSeparationEnd {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set HorizontalSeparationEnd after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{HorizontalSeparationEnd} = $value;
}

sub GetHorizontalSeparationEnd {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{HorizontalSeparationEnd};
}

# --- HorizontalSeparationColumnSeparator ---------------------------------------------------------------

sub SetHorizontalSeparationColumnSeparator {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set HorizontalSeparationColumnSeparator after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{HorizontalSeparationColumnSeparator} = $value;
}

sub GetHorizontalSeparationColumnSeparator {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{HorizontalSeparationColumnSeparator};
}

# --- HorizontalSeparationChar ---------------------------------------------------------------

sub SetHorizontalSeparationChar {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set HorizontalSeparationChar after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{HorizontalSeparationChar} = $value;
}

sub GetHorizontalSeparationChar {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{HorizontalSeparationChar};
}

# --- HorizontalSeparationBoldChar ---------------------------------------------------------------

sub SetHorizontalSeparationBoldChar {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set HorizontalSeparationBoldChar after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{HorizontalSeparationBoldChar} = $value;
}

sub GetHorizontalSeparationBoldChar {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{HorizontalSeparationBoldChar};
}

# --- ConfigureColumnAction ---------------------------------------------------------------

sub SetConfigureColumnAction {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set ConfigureColumnAction after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{ConfigureColumnAction} = $value;
}

sub GetConfigureColumnAction {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{ConfigureColumnAction};
}

# --- ConfigureCompleteAction ---------------------------------------------------------------

sub SetConfigureCompleteAction {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set ConfigureCompleteAction after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{ConfigureCompleteAction} = $value;
}

sub GetConfigureCompleteAction {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{ConfigureCompleteAction};
}

# --- CellOutputAction ---------------------------------------------------------------

sub SetCellOutputAction {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set CellOutputAction after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{CellOutputAction} = $value;
}

sub GetCellOutputAction {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{CellOutputAction};
}

# --- HeaderOutputAction ---------------------------------------------------------------

sub SetHeaderOutputAction {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set HeaderOutputAction after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{HeaderOutputAction} = $value;
}

sub GetHeaderOutputAction {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{HeaderOutputAction};
}

# --- RowOutputAction ---------------------------------------------------------------

sub SetRowOutputAction {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set RowOutputAction after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{RowOutputAction} = $value;
}

sub GetRowOutputAction {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{RowOutputAction};
}

# --- StartTableOutputAction ---------------------------------------------------------------

sub SetStartTableOutputAction {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set StartTableOutputAction after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{StartTableOutputAction} = $value;
}

sub GetStartTableOutputAction {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{StartTableOutputAction};
}

# --- EndTableOutputAction ---------------------------------------------------------------

sub SetEndTableOutputAction {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set EndTableOutputAction after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{EndTableOutputAction} = $value;
}

sub GetEndTableOutputAction {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{EndTableOutputAction};
}

# --- RowGroupChangesAction --------------------------------------------------

sub SetRowGroupChangesAction {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    if ($self->ConfigureIsComplete()) {
        warn ("Cannot set RowGroupChangesAction after configuration has been completed in ".(caller(3))[3]."\n");
        return;
    }
    
    $self->{RowGroupChangesAction} = $value;
}

sub GetRowGroupChangesAction {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{RowGroupChangesAction};
}

# --- Verbose ---------------------------------------------------------------

sub SetVerbose {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;
    
    $self->{Verbose} = $value;
}

sub GetVerbose {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{Verbose};
}

#--------------------------------------------------------------------------------
#
#  Methods
#
#--------------------------------------------------------------------------------

# --- CellOutputActions ---------------------------------------------------------------

sub AddCellOutputAction {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;

    my $cell_output_actions_ref = $self->{CellOutputActions};

    print "### AddCellOutputAction: $value\n" if Verbose($self, 3);
    
    push (@$cell_output_actions_ref, $value);
}

sub GetCellOutputActions {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{CellOutputActions};
}

# --- Store name and action for column ------------------------------------------
sub CC {
    my $self = shift; # instance_ref
    $self->ConfigureColumn(@_);
}

# --- Store name and action for column ------------------------------------------
sub ConfCol {
    my $self = shift; # instance_ref
    $self->ConfigureColumn(@_);
}

# --- Store name and action for column ------------------------------------------
sub ConfigureColumn {
    my $self = $_[0]; # instance_ref
    
    my $action = $self->GetConfigureColumnAction();

    die "no action to store export column defined!" unless ref ($action);
    
    $action->(@_);
}

# --- Create action and trace --------------------------------------------------
sub CreateAction {
    my ($self,        # instance_ref
        $action_str   # will be evaled to a sub { ... }
        ) = @_;

    my $eval_str = 'sub { '.$action_str.' }';

    print "### eval_str = $eval_str\n" if Verbose($self, 3);

    my $sub_ref = eval ($eval_str);

    print "### ref(sub_ref) ".ref($sub_ref) ."\n" if Verbose($self, 3);
    return $sub_ref;
}

# --- Now all columns are defined ---------------------------
sub ConfigureComplete {
    my $self = $_[0]; # instance_ref
    
    my $action = $self->GetConfigureCompleteAction();

    die "no action to complete configuration defined!" unless ref ($action);
    
    $action->(@_);

    $self->{ConfigureComplete} = 1;
}

# --- Now all columns are defined ---------------------------
sub ConfigureIsComplete {
    my $self = $_[0]; # instance_ref

    return $self->{ConfigureComplete};
}

# --- get table header row output ---------------------
sub GetHeaderOutput {
    my ($self,          # instance_ref
        $headers_ref,   # optional: ref to array with header texts
        ) = @_;

    my $action = $self->GetHeaderOutputAction();
    die "No action to output header defined!" unless $action;

    $headers_ref = $self->GetHeaderTextsRef() unless $headers_ref;
    
    $action->($self, $headers_ref);
}

# --- Get output of given row ---------------------
sub GetRowOutput {
    my ($self,      # instance_ref
        $data_ref   # data to give out
        ) = @_;

    my $action = $self->GetRowOutputAction();
    die "No action to output row defined!" unless $action;
    
    $action->($self, $data_ref);
}

# --- Get output of (table) start ---------------------
sub GetOutputStart {
    my ($self,      # instance_ref
        $data_ref   # data to give out
        ) = @_;

    my $action = $self->GetStartTableOutputAction();
    die "No action to start table defined!" unless $action;

    $action->($self, $data_ref);
}

# --- Get output of (table) end ---------------------
sub GetOutputEnd {
    my ($self,      # instance_ref
        $data_ref   # data to give out
        ) = @_;

    my $action = $self->GetEndTableOutputAction();
    die "No action to end table defined!" unless $action;

    $action->($self, $data_ref);
}

# --- Interprete File Parameter and return open file handle ---
sub InterpreteFileParameter {
    my ($self,           # instance_ref
        $file_parameter  # FileHandle or file name for output
        ) = @_;

    my $file_handle;

    if ($file_parameter) {
        my $file_ref = ref($file_parameter);

        if ($file_ref) {
            unless ($file_ref =~ /^FileHandle$/) {
                croak 'Only FileHandle or $file_name_string allowed as file parameter!';
            }
            $file_handle = $file_parameter;
        }
        else {
            $file_handle = FileHandle->new($file_parameter, 'w');
            croak "can't open file '$file_parameter' to write: $!\n" unless $file_handle;
        }
    }
    else {
        $file_handle = *STDOUT;
    }

    return $file_handle;
}


# --- Write out everything in one step ----
sub WriteAll {
    my ($self,           # instance_ref
        $data_rows_ref,  # list of data_rows to give out
        $file_parameter  # Optional: FileHandle or file name for output
        ) = @_;

    my $file_handle = $self->InterpreteFileParameter($file_parameter);

    print $file_handle $self->GetOutputStart();

    my $row_output_action = $self->GetRowOutputAction();

    foreach my $data (@{$data_rows_ref}) {
        print $file_handle $row_output_action->($self, $data);
    }
    
    print $file_handle $self->GetOutputEnd();
}


1;

=head1 NAME

C<Report::Porf::Table::Simple>

Reports for any output format as simple table:
One line in output per data row.

Part of Perl Open Report Framework (Porf).

=head1 Documentation

Use C<Configurator> or C<Framework> of namespace C<Report::Porf::*> to
create Instances, that export data as text, html, csv, LaTeX, for
wikis or Excel.

See Framework.pm for documentation of features and usage.

=head1 LICENSE AND COPYRIGHT

Copyright (c) 2013 by Ralf Peine, Germany.  All rights reserved.

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.6.0 or,
at your option, any later version of Perl 5 you may have available.

=head1 DISCLAIMER OF WARRANTY

This library is distributed in the hope that it will be useful,
but without any warranty; without even the implied warranty of
merchantability or fitness for a particular purpose.

=cut
