# Perl 
#
# Class Report::Porf::Table::Simple::AutoColumnConfigurator
#
# Perl Open Report Framework (Porf)
#
# Configure Report columns automatically
#
# Ralf Peine, Wed Apr 16 19:58:24 2014
#
# More documentation at the end of file
#------------------------------------------------------------------------------

$VERSION = "0.955";

use strict;
use warnings;

#--------------------------------------------------------------------------------
#
#  Report::Porf::Table::Simple::AutoColumnConfigurator
#
#--------------------------------------------------------------------------------

package Report::Porf::Table::Simple::AutoColumnConfigurator;

# --- new Instance, Do NOT call direct!!! -----------------
sub new
{
    my $caller = $_[0];
    my $class  = ref($caller) || $caller;
    
    # let the class go
    my $self = {};
    bless $self, $class;

    $self->{Configurators} = {};
    $self->{max_column_width} = 60;

    return $self;
}

# --- CreateReportConfiguration as string list ---
sub ReportConfigurationAsString {
    my ($self,
	$list_ref,
	$max_rows_to_inspect
	) = @_;
	
	my $config_list = $self->CreateReportConfiguration($list_ref, $max_rows_to_inspect);
	my @result;
	
	foreach my $config (@$config_list) {
		my $line = '$report->CC( ';
		foreach my $key (sort(keys(%$config))) {
			$line .= " $key => ". $config->{$key}.', ';
		}
		$line .= ");";
		push (@result, $line);
	}
	return \@result;
}

# --- CreateReportConfiguration ---
sub CreateReportConfiguration {
    my ($self,
	$list_ref,
	$max_rows_to_inspect
	) = @_;

    return [] unless $list_ref && scalar @$list_ref;

    my $ref_info = ref($list_ref->[0]);
    
    return $self->CreateHashReportConfiguration($list_ref, $max_rows_to_inspect)
	if uc($ref_info) eq 'HASH';
    
    die "cannot create auto configuration for '$ref_info' elements.";
}

# --- crate the default report configuration for a list of hashes --------
sub CreateHashReportConfiguration {
    my ($self,
	$list_ref,
	$max_rows_to_inspect
	) = @_;

    return [] unless $list_ref && scalar @$list_ref;
    
    my @config_list;
    
    my %hash_key_store   = ();
    $max_rows_to_inspect = 10 unless defined $max_rows_to_inspect;
    $max_rows_to_inspect = $#$list_ref if $max_rows_to_inspect == -1;
    
    my $row_count = 0;
    foreach my $data (@$list_ref) {
	foreach my $key (sort(keys(%$data))) {
	    $hash_key_store{$key} = length ($key) unless $hash_key_store{$key};
	    my $text_length = length ($data->{$key});
	    $hash_key_store{$key} = $text_length 
		if $hash_key_store{$key} < $text_length;
	}
	last if $row_count++ >= $max_rows_to_inspect;
    }
    
    foreach my $key (sort(keys(%hash_key_store))) {
	my $width = $hash_key_store{$key};
	$width = $self->{max_column_width} if $width > $self->{max_column_width};
	push (@config_list, {-h => $key, -vn => $key, -w => $width, -a => 'l'});
    }
    
    return \@config_list;
}

# --- create report with automatic configured columns ------------------
sub CreateReport {
    my ($self,
		$list_ref,
		$report_framework,
		$format
	) = @_;

    return undef unless $list_ref && scalar @$list_ref;

    my $first_element = $list_ref->[0];

    die 'Auto reports only possible for hash elements' unless ref $first_element eq 'HASH';

    $report_framework = Report::Porf::Framework::Get() unless $report_framework;
    my $report        = $report_framework->CreateReport($format);
    
    foreach my $config_option (@{$self->CreateReportConfiguration($list_ref)}) {
		$report->CC (%$config_option);
    }

    $report->ConfigureComplete();

    return $report;
}
