# perl
#
# Report example, data rows given as array reference
#
# PORF Perl Open Report Framework
#
# Ralf Peine, Sat May 04 11:25:43 2013
#
#------------------------------------------------------------------------------

use warnings;
use strict;

use Report::Porf::Framework;

sub RunExample {
    my $person_rows   = shift; # Param1: Reference to list of arrays
    my $format        = shift; # Param2: Optional: format of report

    my $count_rows    = scalar @{$person_rows};

    $format = 'text' unless $format;

    # --- Create Report -------------------------------------------------------
    my $report_frame_work   = Report::Porf::Framework::Get();
    my $report_configurator = $report_frame_work->CreateReportConfigurator($format);

    # $report->SetVerbose(3); # uncomment to see infos about configuring phase

    # --- Configure Report ----------------------------------------------------

    $report_configurator->SetAlternateRowColors('#EEEEEE', '#FFFFFF', '#DDDDFF')
        if $report_configurator->can ('SetAlternateRowColors');

    my $report = $report_configurator->CreateAndConfigureReport();

    # store info about data columns
    my $count_idx   = 0;
    my $prename_idx = 1;
    my $surname_idx = 2;
    my $age_idx     = 3;
    my $time_idx    = 4;

    if ( lc($format) eq 'text') {
        $report->SetHeaderRowStart("@");  # overwrite default
        $report->SetHeaderEnd     (" @"); # overwrite default
    }

    # --- configure table -----------------------------------   
    
    $report->ConfigureColumn(-header => 'Count',      -align => 'Center', -value  => sub { return $_[0]->[$count_idx      ]; } );
    $report->ConfigureColumn(-header => 'TimeStamp', -w => 16, -a => 'R', -format => "%.5e sec",
                             -color  => "#CCEECC",                        -v      =>             '$_[0]->['.$time_idx.   ']'   );
    $report->ConfigureColumn(-h      => 'Age',       -w => 15, -a => 'R',
                             -f      => "%.3f years",                     -value_indexed => $age_idx     );
    $report->ConfigureColumn(-h      => 'Prename',   -w => 11, -a => 'l', -val_idx       => $prename_idx );
    $report->ConfigureColumn(-h      => 'Surname',   -width =>  8,        -vi            => $surname_idx );   
    
    $report->ConfigureComplete();

    # --- print out table under own control ----------------------------------- 

    print($report->GetOutputStart());
    
    foreach my $data (@{$person_rows}) {
        print($report->GetRowOutput($data));
    }
    
    print($report->GetOutputEnd()); 
    
    # --- return count of entries ---------------------------   

    return ($count_rows);
}

1;
